<?php
/**
 * Plugin Name: WP Post Content Template | SharedOn ScriptYab.com
 * Plugin URI: http://paulstelzer.de
 * Description: Create a post template from scratch or an exisiting post and save time by using the same structure
 * Version: 1.3.0
 * Author: ScriptYab.com
 * Author URI: http://ScriptYab.com
 * Text Domain: wppct
 * Domain Path: /languages
 * License: Regular/Extended
 * License URI: https://codecanyon.net/licenses
 * @license   GPL-2.0+
 * 
 * 
 * CHANGELOG
 * 2017-05-03 - Version 1.3.0 - Visual Composer now supported
 * 2017-02-01 - Version 1.2.1 - Fixed bug with placeholders
 * 2017-01-31 - Version 1.2.0 - Unlimited placeholders per template
 * 2017-01-29 - Version 1.1.0 - Added docs, filter hooks // Bug fixes
 * 2017-01-24 - Version 1.0.2 - Added Category for templates + searching after templates
 * 2017-01-09 - Version 1.0.1 - Added Roles and Capabilities
*/

namespace InnoMobile;

// Prohibit direct script loading.
defined( 'ABSPATH' ) || die( 'No direct script access allowed!' );

// Define certain plugin variables as constants
define('WPPCT_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));
define('WPPCT_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . WPPCT_PLUGIN_NAME);
define('WPPCT_PLUGIN_URL', WP_PLUGIN_URL . '/' . WPPCT_PLUGIN_NAME);

//Load Classes
require_once("vendor/im-helper/im-helper.php");
require_once("classes/class-wp-post-template.php");

/**
 * WP Post Content Template Plugin Class which holds initialization
 * @since 1.0.0
 * @version 1.2.0
 * @package WPPCT
 */
class WPPCT_PLUGIN {
    /**
     * Plugin Version
     */
    const WPPCT_VERSION_NUM = '1.2.1';
    
    /**
     * Key Plugin Version
     */
    const WPPCT_VERSION_KEY = 'WPPCT_VERSION';
    
    /**
     * WPPCT Post Type slug
     * Should only be modified through the filter hook 'wppct_slug'
     *
     * @since 1.0.0
     * @var string
     */
    public static $wppct_post_type = 'wppct'; 
    
    /**
     * Class Constructor
     */
    public function __construct() {
        // Tags for Translation
        $plugin_name = __('WP Post Content Template', 'wppct');
        $plugin_description = __('Create a post template from scratch or an exisiting post and save time by using the same structure', 'wppct');
        
        //Adding Version to database
        self::update();
        
        //Load translation
        add_action( 'plugins_loaded', array($this, 'load_plugin_textdomain'));
        
        //Enqueue Scripts and Styles
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));

        /**
         * Filter the string that is used as the WPPCT post type slug
         * @since 1.0.0
         * @param string $wppct_post_type wppct post type slug.
         */
        self::$wppct_post_type = apply_filters( 'wppct_slug', self::$wppct_post_type );
        
        //Add WPPCT
        global $WPPCT;
        $WPPCT = new WP_Post_Template(self::$wppct_post_type);

        //Add WPPCT Dashboard
        add_action('init', array($this, 'wppct_meta_box')); 

        //Show admin notice on error
        add_action('init', array($this, 'add_admin_notice'));
        
    }
 
    /**
     * Update Version number of plugin in database VTJOeWFYQjBXV0Zp
     */
    private static function update() {
        $version = get_option(self::WPPCT_VERSION_KEY);
        if(empty($version)) {
            self::plugin_activate();
            add_option(self::WPPCT_VERSION_KEY, self::WPPCT_VERSION_NUM);
        } else {
            $check = version_compare( self::WPPCT_VERSION_NUM, $version);
            if($check > 0) {
                /*
                 *  What should be done if new version of Plugin
                 */
                update_option(self::WPPCT_VERSION_KEY, self::WPPCT_VERSION_NUM);
            }
            
        }
    }
    
    private static function plugin_activate() {
        $roles = array();
        $roles[] = get_role( 'administrator' );
        $roles[] = get_role( 'editor' );
        $roles[] = get_role( 'author' );
        $roles[] = get_role( 'contributor' );

        foreach($roles as $role) {
            if(isset($role->capabilities['edit_posts'])) {
                if($role->capabilities['edit_posts']==1) {
                    $role->add_cap( 'wppct_use_templates' ); 
                }
            }

            if(isset($role->capabilities['publish_posts'])) {
                if($role->capabilities['publish_posts']==1) {
                    $role->add_cap( 'wppct_create_templates' ); 
                }
            }
        } 
    }
    
    /**
     * Load translation for plugin
     */
    public function load_plugin_textdomain() {
        load_plugin_textdomain( 'wppct', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' );
    }
    
    /**
     * Enqueue Scripts and Styles
     */
    public function enqueue_scripts() {   
        wp_enqueue_script( 'wppct_placeholder_script', WPPCT_PLUGIN_URL . '/admin/js/jsPlaceholder.js', array( 'jquery' ), self::WPPCT_VERSION_NUM, true );
        wp_enqueue_script( 'wppct_select2_script', WPPCT_PLUGIN_URL . '/admin/js/jsSelect.js', array( 'jquery' ), self::WPPCT_VERSION_NUM, true );
        
        //Select2 -> https://select2.github.io/
        wp_enqueue_script( 'select2_script', WPPCT_PLUGIN_URL . '/admin/js/select2.min.js', array( 'jquery' ), '4.0.3', true );
        wp_enqueue_style( 'select2_style', WPPCT_PLUGIN_URL . '/admin/css/select2.min.css', '', '4.0.3' );
        
         wp_enqueue_style( 'wppct_styles', WPPCT_PLUGIN_URL . '/admin/css/wppct.css', '', self::WPPCT_VERSION_NUM );
    }
    
    /**
     * Add Metabox
     * @global type $WPPCT
     */
    function wppct_meta_box() {
        $config = array(
          'id'             => 'wppct_meta_box',
          'title'          => __('Template', 'wppct'),
          'post_type'      => self::$wppct_post_type,
          'position'       => 'side',
          'prio'           => 'high',
          'fields'         => array()
        );

        $WPPCT_meta_box = new Meta_Box($config);
        
        $fields = array();
        $fields[] = array(
            'type' => 'text',
            'id' => '_'.self::$wppct_post_type.'_template_name',
            'args' => array( 'title' => __('Template Name', 'wppct') )
        );
        
        $fields[] = array(
            'type' => 'text',
            'id' => '_'.self::$wppct_post_type.'_placeholder',
            'args' => array( 'title' => __('Placeholder', 'wppct')), 
            'repeater' => true
        );
        
        $fields[] = array(
            'type' => 'text',
            'id' => '_'.self::$wppct_post_type.'_category',
            'args' => array( 'title' => __('Template Category', 'wppct'))
        );    
        
        $fields[] = array(
            'type' => 'repeater',
            'id' => '_'.self::$wppct_post_type.'_placeholders',           
            'args' => array('title' => __('Add Placeholder', 'wppct'))
        );
        
        /**
         * Add more fields via filter hook wppct_meta_box_fields
         * @param array fields
         */
        $fields = apply_filters( 'wppct_meta_box_fields', $fields );
        
        $repeater_fields = array();

        foreach($fields as $field) {
            switch($field['type']) {
                case 'text' :
                    if(isset($field['repeater'])) {
                        if($field['repeater']) {
                            $repeater_fields[] = $WPPCT_meta_box->add_text($field['id'],$field['args'],true);
                            break;
                        }
                    }
                    
                    $WPPCT_meta_box->add_text($field['id'],$field['args']);
                    break;
                case 'repeater' :
                    $WPPCT_meta_box->add_repeater($field['id'],$field['args'],$repeater_fields);
                    break;
                default :
                    break;
            }
        }
    }

    /**
     * Add Admin notice on error
     */
    function add_admin_notice() {
        if(isset($_GET['error'])) {
            switch($_GET['error']) {
                case 'notemplate' : 
                    add_notice( __( 'This template could not be found :(', 'wppct' ), 'error' );
                    break;
                case 'notcreated' :
                    add_notice( __( 'The post could not be created :(', 'wppct' ), 'error' );
                    break;
            }
        }
        
    }

}
if(is_admin()) {
    new WPPCT_PLUGIN;
}